//%2006////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2000, 2001, 2002 BMC Software; Hewlett-Packard Development
// Company, L.P.; IBM Corp.; The Open Group; Tivoli Systems.
// Copyright (c) 2003 BMC Software; Hewlett-Packard Development Company, L.P.;
// IBM Corp.; EMC Corporation, The Open Group.
// Copyright (c) 2004 BMC Software; Hewlett-Packard Development Company, L.P.;
// IBM Corp.; EMC Corporation; VERITAS Software Corporation; The Open Group.
// Copyright (c) 2005 Hewlett-Packard Development Company, L.P.; IBM Corp.;
// EMC Corporation; VERITAS Software Corporation; The Open Group.
// Copyright (c) 2006 Hewlett-Packard Development Company, L.P.; IBM Corp.;
// EMC Corporation; Symantec Corporation; The Open Group.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// deal in the Software without restriction, including without limitation the
// rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// sell copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// THE ABOVE COPYRIGHT NOTICE AND THIS PERMISSION NOTICE SHALL BE INCLUDED IN
// ALL COPIES OR SUBSTANTIAL PORTIONS OF THE SOFTWARE. THE SOFTWARE IS PROVIDED
// "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
// ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
//==============================================================================
//
// Authors: David Rosckes (rosckes@us.ibm.com)
//          Bert Rivero (hurivero@us.ibm.com)
//          Chuck Carmack (carmack@us.ibm.com)
//          Brian Lucier (lucier@us.ibm.com)
//
// Modified By: 
//
//%/////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include "QueryIdentifierRep.h"
#include "QueryException.h"
#include <ctype.h>
#include <cstdlib>
PEGASUS_NAMESPACE_BEGIN

QueryIdentifierRep::QueryIdentifierRep(): _isWildcard(false), _isSymbolicConstant(false)
{
	_name = CIMName();
}

QueryIdentifierRep::QueryIdentifierRep(const QueryIdentifierRep* rep)
{
  _symbolicConstant = rep->_symbolicConstant;
  _scope = rep->_scope;
  _indices = rep->_indices;
  _name = rep->_name;
  _isWildcard = rep->_isWildcard;
  _isSymbolicConstant = rep->_isSymbolicConstant;
}

QueryIdentifierRep::~QueryIdentifierRep()
{

}

const CIMName& QueryIdentifierRep::getName()const
{
	return _name;
}

void QueryIdentifierRep::setName(const CIMName& inName)
{
   _name = inName;
}


const String& QueryIdentifierRep::getSymbolicConstantName()const
{
	return _symbolicConstant;
}

const  Array<SubRange>& QueryIdentifierRep::getSubRanges()const
{
	return _indices;
}

Boolean QueryIdentifierRep::isArray()const
{
	return(_indices.size() > 0);
}

Boolean QueryIdentifierRep::isSymbolicConstant()const
{
	return _isSymbolicConstant;
}

Boolean QueryIdentifierRep::isWildcard()const
{
	return _isWildcard;
}

const String& QueryIdentifierRep::getScope()const
{
	return _scope;
}

Boolean QueryIdentifierRep::isScoped()const
{
	return (_scope != String::EMPTY);
}

void QueryIdentifierRep::applyScope(String scope)
{
  _scope = scope;
}

QueryIdentifierRep& QueryIdentifierRep::operator=(const QueryIdentifierRep& rhs){

	if(&rhs != this)
   {
     _symbolicConstant = rhs._symbolicConstant;
     _scope = rhs._scope;
     _indices = rhs._indices;
     _name = rhs._name;
     _isWildcard = rhs._isWildcard;
     _isSymbolicConstant = rhs._isSymbolicConstant;
   }
	return *this;
}

Boolean QueryIdentifierRep::operator==(const CIMName &rhs)const{
	if(_name == rhs) return true;
	return false;
}

Boolean QueryIdentifierRep::operator!=(const CIMName &rhs)const{
	return(!operator==(rhs));
}

Boolean QueryIdentifierRep::operator==(const QueryIdentifierRep &rhs)const
{
	if(_isWildcard && rhs.isWildcard()) return true;
	if(getName() == rhs.getName())
   {
	   if(String::equalNoCase(getScope(), rhs.getScope()))
      {
        if(String::equalNoCase(getSymbolicConstantName(), rhs.getSymbolicConstantName()))
        {
          // compare _indices arrays
          Uint32 size = rhs.getSubRanges().size();
          if(_indices.size() == size)
          {
            Array<SubRange> rhs_SubRanges = rhs.getSubRanges();
            for(Uint32 i = 0; i < size; i++)
            {
              if(_indices[i] != rhs_SubRanges[i])
                return false;
            }
            return true;
          }
        }
      }
	}
	return false;
}

Boolean QueryIdentifierRep::operator!=(const QueryIdentifierRep &rhs)const{
        return(!operator==(rhs));
}

String QueryIdentifierRep::toString()const
{
	if(_isWildcard)
		return "*";
	String s = getScope();
	if(s != String::EMPTY)
		s.append("::");
	s.append(_name.getString());
	if(_isSymbolicConstant){
		s.append("#").append(_symbolicConstant);
		return s;
	}
	if(isArray()){
		s.append("[");
		for(Uint32 i = 0; i < _indices.size(); i++){
			s.append(_indices[i].toString());
			if(i < _indices.size()-1)
				s.append(",");
		}
		s.append("]");
	}
	return s;
}

PEGASUS_NAMESPACE_END
